@@ -76,7 +76,7 @@ from uncertainties import __author__, to_affine_scalar, AffineScalarFunc
 
 # Math functions that have a standard interface: they take
 # one or more float arguments, and return a scalar:
-many_scalar_to_scalar_funcs = []
+many_scalars_to_scalar_funcs = []
 
 # Some functions require a specific treatment and must therefore be
 # excluded from standard wrapping.  Functions
@@ -86,7 +86,7 @@ many_scalar_to_scalar_funcs = []
 num_deriv_funcs = ['fmod', 'gamma', 'isinf', 'isnan',
                    'lgamma', 'trunc']
 
-# Functions that do not belong in many_scalar_to_scalar_funcs, but
+# Functions that do not belong in many_scalars_to_scalar_funcs, but
 # that have a version that handles uncertainties:
 non_std_wrapped_funcs = []
 
@@ -130,9 +130,10 @@ def wraps(wrapper,
 
 # Functions that have singularities (possibly at infinity) benefit
 # from analytical calculations (instead of the default numerical
-# calculation).  Even slowly varying functions (e.g., abs()) yield
-# more precise results when differentiated analytically, because of
-# the loss of precision in numerical calculations.
+# calculation) because their derivatives generally change very fast.
+# Even slowly varying functions (e.g., abs()) yield more precise
+# results when differentiated analytically, because of the loss of
+# precision in numerical calculations.
 
 #def log_1arg_der(x):
 #    """
@@ -235,7 +236,7 @@ for name in dir(math):
     setattr(this_module, name,
             wraps(uncertainties.wrap(func, derivatives), func))
     
-    many_scalar_to_scalar_funcs.append(name)
+    many_scalars_to_scalar_funcs.append(name)
 
 ###############################################################################
     
@@ -313,7 +314,7 @@ def modf(x):
         return (frac_part, int_part)
 modf = uncertainties.set_doc(math.modf.__doc__)(modf)
     
-many_scalar_to_scalar_funcs.append('modf')
+many_scalars_to_scalar_funcs.append('modf')
 
 def ldexp(x, y):
     # The code below is inspired by uncertainties.wrap().  It is
@@ -345,7 +346,7 @@ def ldexp(x, y):
         return math.ldexp(x, y)
 ldexp = uncertainties.set_doc(math.ldexp.__doc__)(ldexp)
     
-many_scalar_to_scalar_funcs.append('ldexp')
+many_scalars_to_scalar_funcs.append('ldexp')
 
 def frexp(x):
     """
@@ -383,5 +384,5 @@ non_std_wrapped_funcs.append('frexp')
 ###############################################################################
 # Exported functions:
 
-__all__ = many_scalar_to_scalar_funcs + non_std_wrapped_funcs
+__all__ = many_scalars_to_scalar_funcs + non_std_wrapped_funcs
 
