@@ -96,13 +96,11 @@ def _derivatives_close(x, y):
     """
 
     # x and y must depend on the same variables:
-    if len(set(x.derivatives)\
-           .symmetric_difference(y.derivatives)):
+    if set(x.derivatives) != set(y.derivatives):
         return False  # Not the same variables
 
-    return all([_numbers_close(x.derivatives[var],
-                               y.derivatives[var])
-                for var in x.derivatives])
+    return all(_numbers_close(x.derivatives[var], y.derivatives[var])
+               for var in x.derivatives)
 
 def test_inverse():
     "Tests of the matrix inverse"
@@ -257,18 +255,6 @@ def test_component_extraction():
     assert type(unumpy.nominal_values(mat)) == numpy.matrix
     
 
-def test_legacy_code():
-
-    "Test of legacy functions"
-    
-    arr_def = ([1, 2], [0.1, 0.01])
-
-    # uarray, nominal_values, std_devs from the uncertainties module
-    # (not the uncertainties.unumpy module):
-    arr = uncertainties.array_u(arr_def)  # Warning is normal
-    assert numpy.all(uncertainties.nominal_values(arr) == arr_def[0])  # Warning is normal
-    assert numpy.all(uncertainties.std_devs(arr) == arr_def[1])  # Warning is normal
-    
 def test_array_comparisons():
     "Test of array and matrix comparisons"
 
