@@ -214,8 +214,9 @@ def wrap_array_func(func):
 
             # Update of the list of variables and associated
             # derivatives, for each element:
-            for (derivative_dict, derivative_value) \
-                  in zip(derivatives.flat, numerical_deriv.flat):
+            for (derivative_dict, derivative_value) in (
+                zip(derivatives.flat, numerical_deriv.flat)):
+                
                 if derivative_value:
                     derivative_dict[var] = derivative_value
 
@@ -581,10 +582,10 @@ def define_vectorized_funcs():
         for f_name in ['acos', 'acosh', 'asin', 'atan', 'atan2', 'atanh'])
 
     new_func_names = [func_name_translations.get(function_name, function_name)
-                      for function_name in umath.many_scalar_to_scalar_funcs]
+                      for function_name in umath.many_scalars_to_scalar_funcs]
         
-    for (function_name, unumpy_name) in \
-        zip(umath.many_scalar_to_scalar_funcs, new_func_names):
+    for (function_name, unumpy_name) in zip(
+        umath.many_scalars_to_scalar_funcs, new_func_names):
 
         # ! The newly defined functions (uncertainties.unumpy.cos, etc.)
         # do not behave exactly like their NumPy equivalent (numpy.cos,
