@@ -1,6 +1,4 @@
-#!/usr/bin/env python
-
-#!! When the documentation below is updated, setup.py should be
+#!! Whenever the documentation below is updated, setup.py should be
 # checked for consistency.
 
 '''
@@ -29,7 +27,7 @@ Examples:
   print x**2  # Square: prints "0.04+/-0.004"
   print sin(x**2)  # Prints "0.0399...+/-0.00399..."
 
-  print x.position_in_sigmas(0.17)  # Prints "-3.0": deviation of -3 sigmas
+  print x.std_score(0.17)  # Prints "-3.0": deviation of -3 sigmas
 
   # Access to the nominal value, and to the uncertainty:
   square = x**2  # Square
@@ -234,10 +232,11 @@ import re
 import math
 from math import sqrt, log  # Optimization: no attribute look-up
 import copy
+import warnings
 
 # Numerical version:
-__version_info__ = (1, 7, 1)
-__version__ = '.'.join(str(num) for num in __version_info__)
+__version_info__ = (1, 9)
+__version__ = '.'.join(map(str, __version_info__))
 
 __author__ = 'Eric O. LEBIGOT (EOL)'
 
@@ -246,7 +245,7 @@ __author__ = 'Eric O. LEBIGOT (EOL)'
 __all__ = [
 
     # All sub-modules and packages are not imported by default,
-    # because NumPy could not be installed:
+    # in particular because NumPy might be unavailable.
 
     'ufloat',  # Main function: returns a number with uncertainty
 
@@ -264,7 +263,7 @@ __all__ = [
     'UFloat',
 
     # Wrapper for allowing non-pure-Python function to handle
-    # quantitites with uncertainties:
+    # quantitities with uncertainties:
     'wrap',
 
     # The documentation for wrap() indicates that numerical
@@ -289,13 +288,140 @@ def set_doc(doc_string):
         return func
     return set_doc_string
 
+# Some types known to not depend on Variable objects are put in
+# CONSTANT_TYPES.  The most common types can be put in front, as this
+# may slightly improve the execution speed.
+CONSTANT_TYPES = (float, int, complex, long)
+
+###############################################################################
+# Utility for issuing deprecation warnings
+
+def deprecation(message):
+    '''
+    Warns the user with the given message, by issuing a
+    DeprecationWarning.
+    '''
+    warnings.warn(message, DeprecationWarning, stacklevel=2)
+
+
+###############################################################################
+
+## Definitions that depend on the availability of NumPy:
+
+
+try:
+    import numpy
+except ImportError:
+    pass
+else:
+
+    # NumPy numbers do not depend on Variable objects:    
+    CONSTANT_TYPES += (numpy.number,)
+    
+    # Entering variables as a block of correlated values.  Only available
+    # if NumPy is installed.
+
+    #! It would be possible to dispense with NumPy, but a routine should be
+    # written for obtaining the eigenvectors of a symmetric matrix.  See
+    # for instance Numerical Recipes: (1) reduction to tri-diagonal
+    # [Givens or Householder]; (2) QR / QL decomposition.
+    
+    def correlated_values(nom_values, covariance_mat, tags=None):
+        """
+        Returns numbers with uncertainties (AffineScalarFunc objects)
+        that correctly reproduce the given covariance matrix, and have
+        the given (float) values as their nominal value.
+
+        The correlated_values_norm() function returns the same result,
+        but takes a correlation matrix instead of a covariance matrix.
+        
+        The list of values and the covariance matrix must have the
+        same length, and the matrix must be a square (symmetric) one.
+
+        The numbers with uncertainties returned depend on newly
+        created, independent variables (Variable objects).
+
+        If 'tags' is not None, it must list the tag of each new
+        independent variable.
+
+        nom_values -- sequence with the nominal (real) values of the
+        numbers with uncertainties to be returned.
+
+        covariance_mat -- full covariance matrix of the returned
+        numbers with uncertainties (not the statistical correlation
+        matrix, i.e., not the normalized covariance matrix). For
+        example, the first element of this matrix is the variance of
+        the first returned number with uncertainty.
+        """
+
+        # If no tags were given, we prepare tags for the newly created
+        # variables:
+        if tags is None:
+            tags = (None,) * len(nom_values)
+
+        # The covariance matrix is diagonalized in order to define
+        # the independent variables that model the given values:
+
+        (variances, transform) = numpy.linalg.eigh(covariance_mat)
+
+        # Numerical errors might make some variances negative: we set
+        # them to zero:
+        variances[variances < 0] = 0.
+        
+        # Creation of new, independent variables:
+
+        # We use the fact that the eigenvectors in 'transform' are
+        # special: 'transform' is unitary: its inverse is its transpose:
+
+        variables = tuple(
+            # The variables represent "pure" uncertainties:
+            Variable(0, sqrt(variance), tag)
+            for (variance, tag) in zip(variances, tags))
+
+        # Representation of the initial correlated values:
+        values_funcs = tuple(
+            AffineScalarFunc(value, dict(zip(variables, coords)))
+            for (coords, value) in zip(transform, nom_values))
+
+        return values_funcs
+
+    __all__.append('correlated_values')
+
+    def correlated_values_norm(values_with_std_dev, correlation_mat,
+                               tags=None):
+        '''
+        Returns correlated values like correlated_values(), but takes
+        instead as input:
+
+        - nominal (float) values along with their standard deviation, and
+        
+        - a correlation matrix (i.e. a normalized covariance matrix
+          normalized with individual standard deviations).
+
+        values_with_std_dev -- sequence of (nominal value, standard
+        deviation) pairs. The returned, correlated values have these
+        nominal values and standard deviations.
+
+        correlation_mat -- correlation matrix (i.e. the normalized
+        covariance matrix, a matrix with ones on its diagonal).
+        '''
+
+        (nominal_values, std_devs) = numpy.transpose(values_with_std_dev)
+
+        return correlated_values(
+            nominal_values,
+            correlation_mat*std_devs*std_devs[numpy.newaxis].T,
+            tags)
+        
+    __all__.append('correlated_values_norm')
+    
 ###############################################################################
 
 # Mathematical operations with local approximations (affine scalar
 # functions)
 
 class NotUpcast(Exception):
-    pass
+    'Raised when an object cannot be converted to a number with uncertainty'
 
 def to_affine_scalar(x):
     """
@@ -312,7 +438,7 @@ def to_affine_scalar(x):
         return x
 
     #! In Python 2.6+, numbers.Number could be used instead, here:
-    if isinstance(x, (float, int, complex, long)):
+    if isinstance(x, CONSTANT_TYPES):
         # No variable => no derivative to define:
         return AffineScalarFunc(x, {})
 
@@ -325,9 +451,6 @@ def partial_derivative(f, param_num):
     Returns a function that numerically calculates the partial
     derivative of function f with respect to its argument number
     param_num.
-
-    The step parameter represents the shift of the parameter used in
-    the numerical approximation.
     """
 
     def partial_derivative_of_f(*args):
@@ -343,7 +466,10 @@ def partial_derivative(f, param_num):
         # shifting it does not suffer from finite precision:
         step = 1e-8*abs(shifted_args[param_num])
         if not step:
-            step = 1e-8  # Arbitrary, but "small" with respect to 1
+            # Arbitrary, but "small" with respect to 1, and of the
+            # order of the square root of the precision of double
+            # precision floats:
+            step = 1e-8
 
         shifted_args[param_num] += step
         shifted_f_plus = f(*shifted_args)
@@ -357,10 +483,11 @@ def partial_derivative(f, param_num):
 
 class NumericalDerivatives(object):
     """
-    Sequence with the successive numerical derivatives of a function.
+    Convenient access to the partial derivatives of a function,
+    calculated numerically.
     """
-    # This is a sequence and not a list because the number of
-    # arguments of the function is not known in advance, in general.
+    # This is not a list because the number of arguments of the
+    # function is not known in advance, in general.
 
     def __init__(self, function):
         """
@@ -374,50 +501,68 @@ class NumericalDerivatives(object):
         """
         return partial_derivative(self._function, n)
   
-def wrap(f, derivatives_funcs=None):
+def wrap(f, derivatives_iter=None):
     """
-    Wraps function f so that, when applied to numbers with
-    uncertainties (AffineScalarFunc objects) or float-like arguments,
-    f returns a local approximation of its values (in the form of an
-    object of class AffineScalarFunc).  In this case, if none of the
-    arguments of f involves variables [i.e. Variable objects], f
-    simply returns its usual result.
+    Wraps a function f into a function that also accepts numbers with
+    uncertainties (UFloat objects) and returns a number with
+    uncertainties.  Doing so may be necessary when function f cannot
+    be expressed analytically (with uncertainties-compatible operators
+    and functions like +, *, umath.sin(), etc.).
 
-    When f is not called on AffineScalarFunc or float-like
-    arguments, the original result of f is returned.
+    f must return a scalar (not a list, etc.).
+    
+    In the wrapped function, the standard Python scalar arguments of f
+    (float, int, etc.) can be replaced by numbers with
+    uncertainties. The result will contain the appropriate
+    uncertainty.
+    
+    If no argument to the wrapped function has an uncertainty, f
+    simply returns its usual, scalar result.
 
-    If supplied, 'derivatives_funcs' is a sequence or iterator that
-    generally contains functions; each successive function is the
-    partial derivatives of f with respect to the corresponding
-    variable (one function for each argument of f, which takes as many
-    arguments as f).  If derivatives_funcs is None, or if
-    derivatives_funcs contains a finite number of elements, then
-    missing derivatives are calculated numerically through
-    partial_derivative().
+    If supplied, derivatives_iter can be an iterable that generally
+    contains functions; each successive function is the partial
+    derivative of f with respect to the corresponding variable (one
+    function for each argument of f, which takes as many arguments as
+    f).  If instead of a function, an element of derivatives_iter
+    contains None, then it is automatically replaced by the relevant
+    numerical derivative; this can be used for non-scalar arguments of
+    f (like string arguments).
 
-    Example: wrap(math.sin) is a sine function that can be applied to
+    If derivatives_iter is None, or if derivatives_iter contains a
+    fixed (and finite) number of elements, then any missing derivative
+    is calculated numerically.
+
+    An infinite number of derivatives can be specified by having
+    derivatives_iter be an infinite iterator; this can for instance
+    be used for specifying the derivatives of functions with a
+    undefined number of argument (like sum(), whose partial
+    derivatives all return 1).
+
+    Example (for illustration purposes only, as
+    uncertainties.umath.sin() runs faster than the examples that
+    follow): wrap(math.sin) is a sine function that can be applied to
     numbers with uncertainties.  Its derivative will be calculated
     numerically.  wrap(math.sin, [None]) would have produced the same
     result.  wrap(math.sin, [math.cos]) is the same function, but with
     an analytically defined derivative.
     """
 
-    if derivatives_funcs is None:
-        derivatives_funcs = NumericalDerivatives(f)
+    if derivatives_iter is None:
+        derivatives_iter = NumericalDerivatives(f)
     else:
         # Derivatives that are not defined are calculated numerically,
         # if there is a finite number of them (the function lambda
         # *args: fsum(args) has a non-defined number of arguments, as
-        # it just performs a sum...
+        # it just performs a sum):
         try:  # Is the number of derivatives fixed?
-            len(derivatives_funcs)
+            len(derivatives_iter)
         except TypeError:
             pass
         else:
-            derivatives_funcs = [
+            derivatives_iter = [
                 partial_derivative(f, k) if derivative is None
                 else derivative
-                for (k, derivative) in enumerate(derivatives_funcs)]
+                for (k, derivative) in enumerate(derivatives_iter)]
 
     #! Setting the doc string after "def f_with...()" does not
     # seem to work.  We define it explicitly:
@@ -434,7 +579,7 @@ def wrap(f, derivatives_funcs=None):
     
     Original documentation:
     %s""" % (f.__name__, f.__doc__))
-    def f_with_affine_output(*args):
+    def f_with_affine_output(*args, **kwargs):
         # Can this function perform the calculation of an
         # AffineScalarFunc (or maybe float) result?
         try:
@@ -442,7 +587,7 @@ def wrap(f, derivatives_funcs=None):
 
         except NotUpcast:
 
-            # This function does not now how to itself perform
+            # This function does not know how to itself perform
             # calculations with non-float-like arguments (as they
             # might for instance be objects whose value really changes
             # if some Variable objects had different values):
@@ -514,14 +659,14 @@ def wrap(f, derivatives_funcs=None):
         # caller to always provide derivatives.  When changing the
         # functions of the math module, this would force this module
         # to know about all the math functions.  Another possibility
-        # would be to force derivatives_funcs to contain, say, the
+        # would be to force derivatives_iter to contain, say, the
         # first 3 derivatives of f.  But any of these two ideas has a
         # chance to break, one day... (if new functions are added to
         # the math module, or if some function has more than 3
         # arguments).
 
         derivatives_wrt_args = []
-        for (arg, derivative) in zip(aff_funcs, derivatives_funcs):
+        for (arg, derivative) in zip(aff_funcs, derivatives_iter):
             derivatives_wrt_args.append(derivative(*args_values)
                                         if arg.derivatives
                                         else 0)
@@ -544,8 +689,15 @@ def wrap(f, derivatives_funcs=None):
         return AffineScalarFunc(f_nominal_value, derivatives_wrt_vars)
 
     # It is easier to work with f_with_affine_output, which represents
-    # a wrapped version of 'f', when it bears the same name as 'f':
+    # a wrapped version of 'f', when it bears the same name as
+    # 'f'.
     f_with_affine_output.__name__ = f.__name__
+    # !! Note: Setting __name__ is however not fully sufficient: the
+    # name f_with_affine_output is stored in
+    # f_with_affine_output.__code__.co_name; being able to change it
+    # would be useful for instance when f_with_affine_output() is
+    # called with unexpected arguments (unexpected keyword argument,
+    # etc.). co_name is read-only, though.
 
     return f_with_affine_output
 
@@ -679,7 +831,7 @@ class AffineScalarFunc(object):
       All the Variable objects on which the function depends are in
       'derivatives'.
 
-    - position_in_sigmas(x): position of number x with respect to the
+    - std_score(x): position of number x with respect to the
       nominal value, in units of the standard deviation.
     """
 
@@ -758,9 +910,6 @@ class AffineScalarFunc(object):
         # vector of that space):
         return self != 0.  # Uses the AffineScalarFunc.__ne__ function
 
-    # Compatibility with Python 3:
-    __bool__ = __nonzero__
-    
     ########################################
     
     ## Logical operators: warning: the resulting value cannot always
@@ -874,6 +1023,14 @@ class AffineScalarFunc(object):
         return self._general_representation(str)
 
     def position_in_sigmas(self, value):
+        '''
+        Wrapper for legacy code.  Obsolete: do not use.  Use std_score
+        instead.
+        '''
+        deprecation("position_in_sigmas is obsolete.  Use std_score instead")
+        return self.std_score(value)
+    
+    def std_score(self, value):
         """
         Returns 'value' - nominal value, in units of the standard
         deviation.
@@ -974,11 +1131,12 @@ def get_ops_with_reflection():
 
     return ops_with_reflection
 
-# Operators that have a reflexion, along with their derivatives:
+# Operators that have a reflection, along with their derivatives:
 _ops_with_reflection = get_ops_with_reflection()
 
 # Some effectively modified operators (for the automated tests):
 _modified_operators = []
+_modified_ops_with_reflection = []
 
 def add_operators_to_AffineScalarFunc():
     """
@@ -996,7 +1154,7 @@ def add_operators_to_AffineScalarFunc():
     ## Operators that return a numerical value:
 
     # Single-argument operators that should be adapted from floats to
-    # AffineScalarFunc objects:
+    # AffineScalarFunc objects, associated to their derivative:
     simple_numerical_operators_derivatives = {
         'abs': lambda x: 1. if x>=0 else -1.,
         'neg': lambda x: -1.,
@@ -1004,17 +1162,20 @@ def add_operators_to_AffineScalarFunc():
         'trunc': lambda x: 0.
         }
 
-    for (op, derivative) in \
-          simple_numerical_operators_derivatives.iteritems():
+    for (op, derivative) in (
+          simple_numerical_operators_derivatives.iteritems()):
         
         attribute_name = "__%s__" % op
         # float objects don't exactly have the same attributes between
         # different versions of Python (for instance, __trunc__ was
         # introduced with Python 2.6):
-        if attribute_name in dir(float):
+        try:
             setattr(AffineScalarFunc, attribute_name,
                     wrap(getattr(float, attribute_name),
                                  [derivative]))
+        except AttributeError:
+            pass
+        else:
             _modified_operators.append(op)
             
     ########################################
@@ -1022,8 +1183,16 @@ def add_operators_to_AffineScalarFunc():
     # Reversed versions (useful for float*AffineScalarFunc, for instance):
     for (op, derivatives) in _ops_with_reflection.iteritems():
         attribute_name = '__%s__' % op
-        setattr(AffineScalarFunc, attribute_name,
-                wrap(getattr(float, attribute_name), derivatives))
+        # float objects don't exactly have the same attributes between
+        # different versions of Python (for instance, __div__ and
+        # __rdiv__ were removed, in Python 3):
+        try:
+            setattr(AffineScalarFunc, attribute_name,
+                    wrap(getattr(float, attribute_name), derivatives))
+        except AttributeError:
+            pass
+        else:
+            _modified_ops_with_reflection.append(op)
 
     ########################################
     # Conversions to pure numbers are meaningless.  Note that the
@@ -1043,6 +1212,10 @@ class Variable(AffineScalarFunc):
     """
     Representation of a float-like scalar random variable, along with
     its uncertainty.
+
+    Objects are meant to represent variables that are independent from
+    each other (correlations are handled through the AffineScalarFunc
+    class).    
     """
 
     # To save memory in large arrays:
@@ -1131,6 +1304,13 @@ class Variable(AffineScalarFunc):
         return (num_repr if ((self.tag is None) or (to_string != repr))
                 else "< %s = %s >" % (self.tag, num_repr))
 
+    def __hash__(self):
+        # All Variable objects are by definition independent
+        # variables, so they never compare equal; therefore, their
+        # id() are therefore allowed to differ
+        # (http://docs.python.org/reference/datamodel.html#object.__hash__):
+        return id(self)
+            
     def __copy__(self):
         """
         Hook for the standard copy module.
@@ -1206,32 +1386,36 @@ def std_dev(x):
 
     return x.std_dev() if isinstance(x, AffineScalarFunc) else 0.
 
-def covariance_matrix(functions):
+def covariance_matrix(nums_with_uncert):
     """
     Returns a matrix that contains the covariances between the given
     sequence of numbers with uncertainties (AffineScalarFunc objects).
     The resulting matrix implicitly depends on their ordering in
-    'functions'.
+    'nums_with_uncert'.
 
     The covariances are floats (never int objects).
 
     The returned covariance matrix is the exact linear approximation
-    result, if the nominal values of the functions and of their
-    variables are their mean.  Otherwise, the returned covariance
-    matrix should be close to it linear approximation value.
+    result, if the nominal values of the numbers with uncertainties
+    and of their variables are their mean.  Otherwise, the returned
+    covariance matrix should be close to its linear approximation
+    value.
+
+    The returned matrix is a list of lists.
     """
-    # See PSI.411.
+    # See PSI.411 in EOL's notes.
 
     covariance_matrix = []
-    for (i1, expr1) in enumerate(functions):
+    for (i1, expr1) in enumerate(nums_with_uncert):
         derivatives1 = expr1.derivatives  # Optimization
         vars1 = set(derivatives1)
         coefs_expr1 = []
-        for (i2, expr2) in enumerate(functions[:i1+1]):
+        for (i2, expr2) in enumerate(nums_with_uncert[:i1+1]):
             derivatives2 = expr2.derivatives  # Optimization
             coef = 0.
             for var in vars1.intersection(derivatives2):
-                # var is a variable common to both functions:
+                # var is a variable common to both numbers with
+                # uncertainties:
                 coef += (derivatives1[var]*derivatives2[var]*var._std_dev**2)
             coefs_expr1.append(coef)
         covariance_matrix.append(coefs_expr1)
@@ -1239,73 +1423,29 @@ def covariance_matrix(functions):
     # We symmetrize the matrix:
     for (i, covariance_coefs) in enumerate(covariance_matrix):
         covariance_coefs.extend(covariance_matrix[j][i]
-                                 for j in range(i+1, len(covariance_matrix)))
+                                for j in range(i+1, len(covariance_matrix)))
 
     return covariance_matrix
 
-# Entering variables as a block of correlated values.  Only available
-# if NumPy is installed.
-
-#! It would be possible to dispense with NumPy, but a routine should be
-# written for obtaining the eigenvectors of a symmetric matrix.  See
-# for instance Numerical Recipes: (1) reduction to tri-diagonal
-# [Givens or Householder]; (2) QR / QL decomposition.
-
 try:
     import numpy
 except ImportError:
     pass
 else:
-    
-    def correlated_values(values, covariance_mat, tags=None):
-        """
-        Returns numbers with uncertainties (AffineScalarFunc objects)
-        that correctly reproduce the given covariance matrix, and have
-        the given values as their nominal value.
+    def correlation_matrix(nums_with_uncert):
+        '''
+        Returns the correlation matrix of the given sequence of
+        numbers with uncertainties, as a NumPy array of floats.
+        '''
 
-        The list of values and the covariance matrix must have the
-        same length, and the matrix must be a square (symmetric) one.
+        cov_mat = numpy.array(covariance_matrix(nums_with_uncert))
 
-        The affine functions returned depend on newly created,
-        independent variables (Variable objects).
-
-        If 'tags' is not None, it must list the tag of each new
-        independent variable.
-        """
-
-        # If no tags were given, we prepare tags for the newly created
-        # variables:
-        if tags is None:
-            tags = (None,) * len(values)
-
-        # The covariance matrix is diagonalized in order to define
-        # the independent variables that model the given values:
-
-        (variances, transform) = numpy.linalg.eigh(covariance_mat)
-
-        # Numerical errors might make some variances negative: we set
-        # them to zero:
-        variances[variances < 0] = 0.
+        std_devs = numpy.sqrt(cov_mat.diagonal())
         
-        # Creation of new, independent variables:
-
-        # We use the fact that the eigenvectors in 'transform' are
-        # special: 'transform' is unitary: its inverse is its transpose:
-
-        variables = tuple(
-            # The variables represent uncertainties only:
-            Variable(0, sqrt(variance), tag)
-            for (variance, tag) in zip(variances, tags))
-
-        # Representation of the initial correlated values:
-        values_funcs = tuple(
-            AffineScalarFunc(value, dict(zip(variables, coords)))
-            for (coords, value) in zip(transform, values))
-
-        return values_funcs
-
-    __all__.append('correlated_values')
+        return cov_mat/std_devs/std_devs[numpy.newaxis].T
 
+    __all__.append('correlation_matrix')
+        
 ###############################################################################
 # Parsing of values with uncertainties:
 
@@ -1440,6 +1580,7 @@ def ufloat(representation, tag=None):
         1234567(1.2)
         12.345(15)
         -12.3456(78)e-6
+        12.3(0.4)e-5        
         0.29
         31.
         -31.
@@ -1467,79 +1608,14 @@ def ufloat(representation, tag=None):
         representation = str_to_number_with_uncert(representation)
         
     #! The tag is forced to be a string, so that the user does not
-    # create a Variable(2.5, 0.5) in order to represent 2.5
-    # +/- 0.5.  Forcing 'tag' to be a string prevents errors from being
-    # considered as tags, here:
-
-    #! 'unicode' is removed in Python3:
+    # create a Variable(2.5, 0.5) in order to represent 2.5 +/- 0.5.
+    # Forcing 'tag' to be a string prevents numerical uncertainties
+    # from being considered as tags, here:
     if tag is not None:
-        assert ((type(tag) is str) or (type(tag) is unicode)), \
-               "The tag can only be a string."
+        #! 'unicode' is removed in Python3:
+        assert isinstance(tag, (str, unicode)), "The tag can only be a string."
 
-    #! init_args must contain all arguments, here:
+    #! The special ** syntax is for Python 2.5 and before (Python 2.6+
+    # understands tag=tag):
     return Variable(*representation, **{'tag': tag})
 
-###############################################################################
-# Support for legacy code (will be removed in the future):
-    
-def NumberWithUncert(*args):
-    """
-    Wrapper for legacy code.  Obsolete: do not use.  Use ufloat
-    instead.
-    """
-    import warnings
-    warnings.warn("NumberWithUncert is obsolete."
-                  "  Use ufloat instead.", DeprecationWarning,
-                  stacklevel=2)
-    return ufloat(*args)
-
-def num_with_uncert(*args):
-    """
-    Wrapper for legacy code.  Obsolete: do not use.  Use ufloat
-    instead.
-    """
-    import warnings
-    warnings.warn("num_with_uncert is obsolete."
-                  "  Use ufloat instead.", DeprecationWarning,
-                  stacklevel=2)
-    return ufloat(*args)
-
-def array_u(*args):
-    """
-    Wrapper for legacy code.  Obsolete: do not use.  Use
-    unumpy.uarray instead.
-    """
-    import warnings
-    warnings.warn("uncertainties.array_u is obsolete."
-                  " Use uncertainties.unumpy.uarray instead.",
-                  DeprecationWarning,
-                  stacklevel=2)
-    import uncertainties.unumpy
-    return uncertainties.unumpy.uarray(*args)
-
-def nominal_values(*args):
-    """
-    Wrapper for legacy code.  Obsolete: do not use.  Use
-    unumpy.nominal_values instead.
-    """
-    import warnings
-    warnings.warn("uncertainties.nominal_values is obsolete."
-                  "  Use uncertainties.unumpy.nominal_values instead.",
-                  DeprecationWarning,
-                  stacklevel=2)
-    import uncertainties.unumpy
-    return uncertainties.unumpy.nominal_values(*args)
-
-def std_devs(*args):
-    """
-    Wrapper for legacy code.  Obsolete: do not use.  Use ufloat
-    instead.
-    """
-    import warnings
-    warnings.warn("uncertainties.std_devs is obsolete."
-                  "  Use uncertainties.unumpy.std_devs instead.",
-                  DeprecationWarning,
-                  stacklevel=2)
-    import uncertainties.unumpy
-    return uncertainties.unumpy.std_devs(*args)
-
