@@ -9,17 +9,17 @@ These tests can be run through the Nose testing framework.
 from __future__ import division
 
 # 3rd-party modules:
-import numpy
+try:
+    import numpy
+except ImportError:
+    import sys
+    sys.exit()  # There is no reason to test the interface to NumPy    
 
 # Local modules:
 import uncertainties
-from uncertainties import ufloat
-from uncertainties import unumpy
-from . import core
-
-from .. import test_uncertainties
+from uncertainties import ufloat, unumpy, test_uncertainties
+from uncertainties.unumpy import core
 from uncertainties.test_uncertainties import _numbers_close, matrices_close
-
 from uncertainties import __author__
 
 def test_numpy():
@@ -82,6 +82,10 @@ def test_matrix():
 
     assert type(m[0, 0]) == uncertainties.Variable
 
+    # Test of scalar multiplication, both sides:
+    3*m
+    m*3
+
 def _derivatives_close(x, y):
     """
     Returns True iff the AffineScalarFunc objects x and y have
@@ -90,13 +94,11 @@ def _derivatives_close(x, y):
     """
 
     # x and y must depend on the same variables:
-    if len(set(x.derivatives)\
-           .symmetric_difference(y.derivatives)):
+    if set(x.derivatives) != set(y.derivatives):
         return False  # Not the same variables
 
-    return all( _numbers_close(x.derivatives[var],
-                               y.derivatives[var])
-                for var in x.derivatives)
+    return all(_numbers_close(x.derivatives[var], y.derivatives[var])
+               for var in x.derivatives)
 
 def test_inverse():
     "Tests of the matrix inverse"
@@ -251,18 +253,6 @@ def test_component_extraction():
     assert type(unumpy.nominal_values(mat)) == numpy.matrix
     
 
-def test_legacy_code():
-
-    "Test of legacy functions"
-    
-    arr_def = ([1, 2], [0.1, 0.01])
-
-    # uarray, nominal_values, std_devs from the uncertainties module
-    # (not the uncertainties.unumpy module):
-    arr = uncertainties.array_u(arr_def)  # Warning is normal
-    assert numpy.all(uncertainties.nominal_values(arr) == arr_def[0])  # Warning is normal
-    assert numpy.all(uncertainties.std_devs(arr) == arr_def[1])  # Warning is normal
-    
 def test_array_comparisons():
     "Test of array and matrix comparisons"
 
